@@ -8,12 +8,13 @@ use crate::VarZeroVec;
 use crate::ZeroVec;
 use std::cmp::Ordering;
 
-/// Trait marking types which are allowed to be keys or values in [`ZeroMap`].
+/// Trait marking types which are allowed to be keys or values in [`ZeroMap`](super::ZeroMap).
 ///
 /// Users should not be calling methods of this trait directly, however if you are
 /// implementing your own [`AsULE`] or [`AsVarULE`] type you may wish to implement
 /// this trait.
 // this lifetime should be a GAT on Container once that is possible
+#[allow(clippy::upper_case_acronyms)] // KV is not an acronym
 pub trait ZeroMapKV<'a>: Sized {
     /// The container that can be used with this type: [`ZeroVec`] or [`VarZeroVec`].
     type Container: ZeroVecLike<'a, Self, NeedleType = Self::NeedleType, GetType = Self::GetType>
